#!/usr/bin/python3

import base64
import chevron
import configparser
import hashlib
import html.parser
import io
import json
import urllib
import urllib.request
import urllib.parse

template_string = r'''
<!DOCTYPE html>
<html>
<head>
	<meta charset="UTF-8">
	<title>TF2 Attribute List</title>
	
	<style>
	html { scroll-behavior: smooth; }
	body {
		font-family: Verdana,Arial,sans-serif;
		font-size: 0.75em;
		
		background-color: #111;
		color: #FFF;
	}
	td { padding: 0.25em; }
	a { color: #1eb; }
	.attribute {
		vertical-align: top;
	}
	.header {
		font-weight: bold;
		position: sticky;
		top: 0;
		background-color: #111;
	}
	.value_type {
		overflow: hidden;
	}
	.footer {
		margin-top: 2.5em;
		text-align: center;
	}
	.positive { color: #28F; }
	.negative { color: #F00; }
	
	tr:nth-child(even) { background: #222; }
	tr:hover { background: #333; }
	
	tr:target {
		box-shadow: 0 0 1em .25em #abb;
		position: relative;
	}
	.id a {
		color: inherit;
		text-decoration: none;
		display: block;
	}
	</style>
</head>
<body>
	<h1>List of Team Fortress 2 attributes:</h1>
	
	<p>Page generated for schema dated {{update_time}}.  <a href="" download>Click here to download this page for offline viewing.</a></p>
	
	<table class="attribute-list">
		<thead>
			<tr class="attribute header">
				<th class="id">ID</th>
				<th class="name">Name</th>
				<th class="description">Description</th>
				<th class="is_hidden">Hidden</th>
				<th class="value_type">Value Type</th>
				<th class="class_name">Class</th>
			</tr>
		</thead>
		<tbody>
			{{#result.attributes}}
			<tr class="attribute entry" id="{{defindex}}">
				<td class="id"><a href="#{{defindex}}">{{defindex}}</a></td>
				<td class="name {{effect_type}}">{{name}}</td>
				<td class="description">{{description_string}}</td>
				<td class="is_hidden">{{#hidden}}true{{/hidden}}{{^hidden}}false{{/hidden}}</td>
				<td class="value_type">{{description_format}}</td>
				<td class="class_name">{{attribute_class}}</td>
			</tr>
			{{/result.attributes}}
		</tbody>
	</table>
	<div class="footer">
		<div>Generated with <a href="https://git.csrd.science/nosoop/py-tf2-schema-renderer">this script</a>.</div>
		<div>Table sorting provided by <a href="https://github.com/tristen/tablesort">tablesort</a>.</div>
	</div>
	
	<!--
	  - table sorting functionality
	  - the page degrades gracefully without JS, but sorting is pretty nice to have
	  -->
	{{#inline_scripts}}
	<script src="https://unpkg.com/tablesort@5.0.2/dist/tablesort.min.js"
			integrity="sha384-MAoWp5qWRu0089eW0evm9gRng72sxH0daE2mkSXN8RoQ99hG/x+Wi8GxUbrgbYLp"
			crossorigin="anonymous"></script>
	<script src="https://unpkg.com/tablesort@5.0.2/dist/sorts/tablesort.number.min.js"
			integrity="sha384-YPqBWGhq2JCwnOo6VsMYgE5XvUqQrzskaIGxc7CByfV6o9viRVq8uAhCs/0iPQBw"
			crossorigin="anonymous"></script>
	{{/inline_scripts}}
	<script>
		new Tablesort(document.querySelector('.attribute-list'));
		window.onload = window.onhashchange = () => {
			if (location.hash) {
				document.getElementById(location.hash.slice(1)).scrollIntoView({
					behavior: "auto", block: "center", inline: "center"
				});
			}
		};
	</script>
</body>
</html>
'''

def get_schema_overview(params):
	opts = dict(params.items())
	url = "https://api.steampowered.com/IEconItems_440/GetSchemaOverview/v0001/"
	with urllib.request.urlopen(f"{url}?{urllib.parse.urlencode(opts)}") as r:
		return { **json.load(r), 'update_time': r.info().get('last-modified') }

def inline_scripts(text, render):
	"""
	Helper function to parse script elements, download the scripts and inline them.
	"""
	class ScriptElementParser(html.parser.HTMLParser):
		def __init__(self):
			super(ScriptElementParser, self).__init__()
			self.scripts = []
			self.current = []
		
		def handle_starttag(self, tag, attrs):
			if tag == 'script':
				self.current.append(dict(attrs))
		
		def handle_endtag(self, tag):
			self.scripts.append(self.current.pop())
	
	parser = ScriptElementParser()
	parser.feed(text)
	
	f = io.StringIO()
	for script in parser.scripts:
		with urllib.request.urlopen(script['src']) as r:
			data = r.read()
			if 'integrity' in script:
				ct, hash = script['integrity'].split('-', 1)
				h = hashlib.new(ct)
				h.update(data)
				if base64.b64encode(h.digest()).decode('ascii') != hash:
					raise ValueError(f"{script['src']} failed subresource integrity check {script['integrity']} (got {base64.b64encode(h.digest())})")
			f.write(f"<script>\n/* {script['src']} */\n{data.decode('utf8')}\n</script>\n")
	
	return f.getvalue()

config = configparser.ConfigParser()
config.read('config.ini', encoding = "utf8")

# tf_schema_overview.json is the WebAPI call to IEconItems_440/GetSchemaOverview
with open('attributes.html', 'wt', encoding = 'utf8') as f:
	f.write(chevron.render(template_string, {
		**get_schema_overview(config['DEFAULT']),
		'inline_scripts': inline_scripts,
	}))
